@@ -2,7 +2,7 @@ package Test2::Hub::Interceptor;
 use strict;
 use warnings;
 
-our $VERSION = '1.302049';
+our $VERSION = '1.302125';
 
 
 use Test2::Hub::Interceptor::Terminator();
@@ -10,10 +10,18 @@ use Test2::Hub::Interceptor::Terminator();
 BEGIN { require Test2::Hub; our @ISA = qw(Test2::Hub) }
 use Test2::Util::HashBase;
 
+sub init {
+    my $self = shift;
+    $self->SUPER::init;
+    $self->{+NESTED} = 0;
+}
+
 sub inherit {
     my $self = shift;
     my ($from, %params) = @_;
 
+    $self->{+NESTED} = 0;
+
     if ($from->{+IPC} && !$self->{+IPC} && !exists($params{ipc})) {
         my $ipc = $from->{+IPC};
         $self->{+IPC} = $ipc;
@@ -24,7 +32,15 @@ sub inherit {
 sub terminate {
     my $self = shift;
     my ($code) = @_;
-    die bless(\$code, 'Test2::Hub::Interceptor::Terminator');
+
+    eval {
+        no warnings 'exiting';
+        last T2_SUBTEST_WRAPPER;
+    };
+    my $err = $@;
+
+    # Fallback
+    die bless(\$err, 'Test2::Hub::Interceptor::Terminator');
 }
 
 1;
@@ -62,7 +78,7 @@ F<http://github.com/Test-More/test-more/>.
 
 =head1 COPYRIGHT
 
-Copyright 2016 Chad Granum E<lt>exodist@cpan.orgE<gt>.
+Copyright 2018 Chad Granum E<lt>exodist@cpan.orgE<gt>.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
